<?php

function services_sso_client_propagate_batch_process(&$context) {
  if (!isset($context['sandbox']['progress'])) {
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['offset'] = 0;
    $context['sandbox']['max'] = 0;
  }

  // Create an API account session so we can use it.
  $session_data = services_sso_client_propagate_api_account();

  if (!empty($session_data)) {
    $users = TRUE;
    $accounts = array();

    // For this example, we decide that we can safely process
    // some flags at a time without a timeout.
    $limit = variable_get('services_sso_client_propagate_batch_num', 2);

    // Grab some accounts from SSO server.
    if ($users = services_sso_client_propagate_get_accounts($session_data, $context['sandbox']['offset'])) {
      $accounts = array_merge($accounts, $users);
    }
  }

  if (!empty($accounts)) {
    $ignore = _services_sso_client_ignore_accounts();

    foreach ($accounts as $account) {
      if (!($local_account = user_external_load($account->uid))) {
        if (!in_array($account->name, $ignore) && !empty($account->uid)) {
          // Copy the account to local.
          $local_account = services_sso_client_propagate_copy_account($session_data, $account);

          // Break of something went wrong.
          if (!$local_account) {
            watchdog('services_sso_client', t('Couldn\'t create account for SSO user: ' . $account->name));
            break;
          }

          else {
            // Store some result for post-processing in the finished callback.
            $context['results'][] = check_plain($account->name);
          }
        }
      }

      // Update our progress information.
      $context['sandbox']['progress']++;
      $context['sandbox']['offset']++;
      $context['message'] = t('Now processing %flag', array('%flag' => $flag->fid . ': ' . $flag->title));
    }
  }

  // If we didn't manage to grab any accounts, we are done.
  else {
    // Inform the batch engine that we are not finished,
    // and provide an estimation of the completion level we reached.
    $context['finished'] = 1;
  }

  // This will destroy the API account session.
  services_sso_client_propagate_api_account($session_data);
}

/**
 * Batch 'finished' callback
 */
function services_sso_client_propagate_batch_finished($success, $results, $operations) {
  if ($success) {
    // Here we do something meaningful with the results.
    $message = count($results) .' processed.';
    if (count($results) < 10) {
      $message .= theme('item_list', $results);
    }
  }
  else {
    // An error occurred.
    // $operations contains the operations that remained unprocessed.
    $error_operation = reset($operations);
    $message = t('An error occurred while processing %error_operation with arguments: @arguments', array('%error_operation' => $error_operation[0], '@arguments' => print_r($error_operation[1], TRUE)));
  }
  drupal_set_message($message);
}